/*
 * linux/include/linux/beos_fs.h
 *
 * Copyright (C) 1999 Makoto Kato (m_kato@ga2.so-net.ne.jp)
 */

#ifndef _LINUX_BEOS_FS
#define _LINUX_BEOS_FS

#include <linux/types.h>
//#include <linux/fs.h>

/*
 * for debug
 */
/* #define BEOS_DEBUG */

#ifdef BEOS_DEBUG
#define BEOS_OUTPUT(x)        printk x
#define BEOS_DUMP_SUPER_BLOCK beos_dump_super_block
#define BEOS_DUMP_INODE       beos_dump_inode
#define BEOS_DUMP_INDEX_ENTRY beos_dump_index_entry
#define BEOS_DUMP_INDEX_NODE  beos_dump_index_node
#define BEOS_DUMP_INODE_ADDR  beos_dump_inode_addr
#else
#define BEOS_OUTPUT(x)
#define BEOS_DUMP_SUPER_BLOCK(x)
#define BEOS_DUMP_INODE(x)
#define BEOS_DUMP_INDEX_ENTRY(x)
#define BEOS_DUMP_INDEX_NODE(x)
#define BEOS_DUMP_INODE_ADDR(x)
#endif


/*
 * Flags of superblock
 */

#define BEOS_CLEAN 0x434c454e
#define BEOS_DIRTY 0x44495254

/*
 * Max name length of BEOS
 */

#define BEOS_NAME_LEN 255
#define BEOS_SYMLINK_LEN 160

/*
 * Magic headers of BEOS's superblock, inode and index
 */

#define BEOS_SUPER_BLOCK_MAGIC1 0x42465331 /* BEOS1 */
#define BEOS_SUPER_BLOCK_MAGIC2 0xdd121031
#define BEOS_SUPER_BLOCK_MAGIC3 0x15b6830e

#define BEOS_INODE_MAGIC1 0x3bbe0ad9

#define BEOS_INDEX_MAGIC 0x69f6c2e8

#define BEOS_SUPER_MAGIC BEOS_SUPER_BLOCK_MAGIC1


#define BEOS_NUM_DIRECT_BLOCKS 12
#define BEOS_NAME_LENGTH 32

/*
 * BEOS filesystem type
 */

#define BEOS_PPC 1
#define BEOS_X86 2

/*
 * default gid and uid
 */

#define BEOS_DEFAULT_GID 0
#define BEOS_DEFAULT_UID 0

/*
 * Flags of inode
 */

#define BEOS_INODE_IN_USE      0x00000001
#define BEOS_ATTR_INODE        0x00000004
#define BEOS_INODE_LOGGED      0x00000008
#define BEOS_INODE_DELETED     0x00000010

#define BEOS_PERMANENT_FLAG    0x0000ffff

#define BEOS_INODE_NO_CREATE   0x00010000
#define BEOS_INODE_WAS_WRITTEN 0x00020000
#define BEOS_NO_TRANSACTION    0x00040000


#define BEOS_IADDR2INO(iaddr,sb) \
	(((iaddr)->allocation_group << (sb)->ag_shift) + (iaddr)->start)
#define BEOS_INODE2INO(inode) \
	(((inode)->u.beos_i.i_inode_num.allocation_group << \
		(inode)->i_sb->u.beos_sb.ag_shift) \
	+ (inode)->u.beos_i.i_inode_num.start)

#define BEOS_BLOCK_PER_INODE(sb) \
	((sb)->u.beos_sb.block_size / sizeof(beos_inode_addr))

#define BEOS_IS_EMPTY_IADDR(iaddr) \
	((!(iaddr)->allocation_group) && (!(iaddr)->start) && (!(iaddr)->len))

#define BEOS_TYPE(sb) \
	((sb)->u.beos_sb.mount_opts.beos_type)

/* 
 * special type of BEOS
 */

typedef long long	beos_off_t;
typedef long long	beos_bigtime_t;
typedef void		beos_binode_etc;


/* Block runs */
typedef struct _beos_block_run {
	__u32	allocation_group;
	__u16	start;
	__u16	len;
} beos_block_run;


typedef beos_block_run	beos_inode_addr;


/*
 * The Superblock Structure
 */

typedef struct _beos_super_block {
	char	name[BEOS_NAME_LENGTH];
	__u32	magic1;
	__u32	fs_byte_order;

	__u32	block_size;
	__u32	block_shift;

	beos_off_t  num_blocks;
	beos_off_t  used_blocks;

	__u32          inode_size;

	__u32          magic2;
	__u32          blocks_per_ag;
	__u32          ag_shift;
	__u32          num_ags;

	__u32          flags;

	beos_block_run  log_blocks;
	beos_off_t      log_start;
	beos_off_t      log_end;

	__u32          magic3;
	beos_inode_addr root_dir;
	beos_inode_addr indices;

	__u32          pad[8];
} __attribute__ ((packed)) beos_super_block;


typedef struct _beos_data_stream {
	beos_block_run	direct[BEOS_NUM_DIRECT_BLOCKS];
	beos_off_t	max_direct_range;
	beos_block_run	indirect;
	beos_off_t	max_indirect_range;
	beos_block_run	double_indirect;
	beos_off_t	max_double_indirect_range;
	beos_off_t	size;
} __attribute__ ((packed)) beos_data_stream;


/* Attribute */
typedef struct _beos_small_data {
	__u32	type;
	__u16	name_size;
	__u16	data_size;
	char	name[1];
} __attribute__ ((packed)) beos_small_data;


/* Inode structure */
typedef struct _beos_inode {
	__u32           magic1;
	beos_inode_addr  inode_num;
	__u32           uid;
	__u32           gid;
	__u32           mode;
	__u32           flags;
	beos_bigtime_t   create_time;
	beos_bigtime_t   last_modified_time;
	beos_inode_addr  parent;
	beos_inode_addr  attributes;
	__u32           type;

	__u32           inode_size;
	__u32           etc; /* not use */

	union {
		beos_data_stream	 datastream;
		char             symlink[BEOS_SYMLINK_LEN];
	} data;
	__u32            pad[4]; /* not use */
} __attribute__ ((packed)) beos_inode;


typedef struct _beos_index_entry {
	__u32      magic;
	__u32      node_size;
	__u32      max_number_of_levels;
	__u32      data_type;
	beos_off_t  root_node_pointer;
	beos_off_t  free_node_pointer;
	beos_off_t  maximum_size;
} __attribute__ ((packed)) beos_index_entry;
	

typedef struct _beos_index_node {
	beos_off_t left;
	beos_off_t right;
	beos_off_t overflow;
	__u16     all_key_count;
	__u16     all_key_length;
} __attribute__ ((packed)) beos_index_node;


typedef struct _beos_mount_options {
	gid_t	gid;
	uid_t	uid;
	int	beos_type;
	char *  iocharset;
} beos_mount_options;


#ifdef __KERNEL__
/*
 * Function prototypes
 */

/* file.c */
extern int beos_read (struct inode *, struct file *, char *, int);
extern beos_inode_addr beos_read_data_stream (struct super_block *,
	beos_data_stream *, int *);

/* inode.c */
extern int beos_bmap (struct inode *, int);
extern void beos_read_inode (struct inode *);
extern struct buffer_head * beos_bread (struct inode *);
extern struct buffer_head * beos_bread2 (struct super_block *, beos_inode_addr);
extern void beos_write_inode (struct inode *);
extern int beos_sync_inode (struct inode *);
extern void beos_convert_inodeaddr (
		int fstype, beos_inode_addr * iaddr, beos_inode_addr * out);

/* namei.c */
extern void beos_release (struct inode *, struct file *);
//extern int beos_lookup (struct inode *, struct dentry *);
extern struct dentry * beos_lookup (struct inode *, struct dentry *);

/* super.c */
extern void beos_put_super (struct super_block *);
extern int beos_remount (struct super_block *, int *, char *);
extern struct super_block * beos_read_super (struct super_block *,void *,int);
extern int init_beos_fs(void);
extern int beos_statfs (struct super_block *, struct statfs *, int);

/* index.c */
extern char * beos_get_key_from_index_node (beos_index_node *, int *, int,
	char *, int *, beos_off_t *);
extern struct buffer_head * beos_read_index_node (beos_inode_addr,
	struct super_block *, int, beos_off_t *);

/* debug.c */
extern void beos_dump_super_block (beos_super_block *);
extern void beos_dump_inode (beos_inode *);
extern void beos_dump_index_entry (beos_index_entry *);
extern void beos_dump_index_node (beos_index_node *);

/* util.c */
extern char * beos_utf2nls (char *, int, char **, int *, struct super_block *);
extern char * beos_nls2utf (unsigned char *, int, char **, int *, struct super_block *);

/*
 * Inodes and files operations
 */

/* dir.c */
extern struct inode_operations beos_dir_inode_operations;

/* file.c */
extern struct inode_operations beos_file_inode_operations;

/* symlink.c */
extern struct inode_operations beos_symlink_inode_operations;

#endif  /* __KERNEL__ */

#endif
